/**
 * Generates THREAD_NUM threads, each of them hitting (allocating,releasing) a common PidTable.
 * It could make use of StoYield to generate richer interleavings.
 * 
 * Author: Nicolás Wolovick (nicolasw@famaf.unc.edu.ar)
 */

import java.util.Vector;


/* Thread job */
final class PidTableUse extends Thread {		   
	private static final int CYCLE_NUM = 32;
	private PidTable<Object> pt = null;
	private int id;

	public PidTableUse(int id, PidTable<Object> pt) {
		this.pt = pt;
		this.id = id;
		this.setName("Thread "+Integer.toString(id));
	}

	public void run() {
		System.out.println("PidTableUse " + id + " start");
		int cycle = 0;
		int index = 0;
		for(cycle=0; cycle<CYCLE_NUM; cycle++) {
			index = pt.alloc(this);
			if (index==pt.capacity()) {
				System.out.println("PidTableUse:" + id + ", cycle:" + cycle + " full table");
			} else {
				System.out.println("PidTableUse:" + id + ", cycle:" + cycle + " got index " + index);
				/* doing something super interesting with pt[index] */
				System.out.println("PidTableUse:" + id + ", cycle:" + cycle + " return index " + index);
				pt.release(index);
			}
		}
		System.out.println("PidTableUse " + id + " end");
	}
}


/* Tester of PidTable with a main() */
public class PidTableTest {

	private static final int THREAD_NUM = 1;
	private static final int TABLE_SIZE = 16;

	public static void main(String[] args) {
		PidTable<Object> pt = new PidTable<Object>(TABLE_SIZE);
		Vector<PidTableUse> threads = new Vector<PidTableUse>();
		long time;
		int i = 0;
		for(i=0; i<THREAD_NUM; i++) {
			threads.add(new PidTableUse(i,pt));
		}
		time = System.currentTimeMillis();
		for (Thread thread: threads) {
			thread.start();
		}
		for (Thread thread: threads) {
			try {
				thread.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		System.out.println("Time: " + (System.currentTimeMillis() - time) + " milisec" );
	}
}
